@@ -98,10 +98,58 @@ class Shape:
         return None
 
 
+# Map PostScript fontnames to Pango description strings
+# See also:
+# - https://graphviz.org/docs/attrs/fontname/
+# - https://gitlab.com/graphviz/graphviz/-/blob/main/lib/common/ps_font_equiv.h
+# - https://docs.gtk.org/Pango/type_func.FontDescription.from_string.html
+# XXX: Ideally we'd call FontConfig to do this for us, but there seems to be no
+# easy way to do that
+fontname_map = {
+    # PostScript Level 1
+    'Courier':                       'Courier',
+    'Courier-Bold':                  'Courier Bold',
+    'Courier-Oblique':               'Courier Oblique',
+    'Courier-BoldOblique':           'Courier Bold Oblique',
+    'Helvetica':                     'Helvetica',
+    'Helvetica-Bold':                'Helvetica Bold',
+    'Helvetica-Oblique':             'Helvetica Oblique',
+    'Helvetica-BoldOblique':         'Helvetica Bold Oblique',
+    'Times-Roman':                   'Times Roman',
+    'Times-Bold':                    'Times Bold',
+    'Times-Italic':                  'Times Italic',
+    'Times-BoldItalic':              'Times Bold Italic',
+    'Symbol':                        'Symbol',
+
+    # PostScript Level 2
+    'AvantGarde-Book':               'AvantGarde Book',
+    'AvantGarde-BookOblique':        'AvantGarde Book Oblique',
+    'AvantGarde-Demi':               'AvantGarde DemiBold',
+    'AvantGarde-DemiOblique':        'AvantGarde DemiBold Oblique',
+    'Bookman-Demi':                  'Bookman DemiBold',
+    'Bookman-DemiItalic':            'Bookman DemiBold Italic',
+    'Bookman-Light':                 'Bookman Light',
+    'Bookman-LightItalic':           'Bookman Light Italic',
+    'Helvetica-Narrow':              'Helvetica Condensed',
+    'Helvetica-Narrow-Bold':         'Helvetica Condensed Bold',
+    'Helvetica-Narrow-Oblique':      'Helvetica Condensed Oblique',
+    'Helvetica-Narrow-BoldOblique':  'Helvetica Condensed Bold Oblique',
+    'NewCenturySchlbk-Bold':         'NewCenturySchlbk Bold',
+    'NewCenturySchlbk-BoldItalic':   'NewCenturySchlbk Bold Italic',
+    'NewCenturySchlbk-Italic':       'NewCenturySchlbk Italic',
+    'NewCenturySchlbk-Roman':        'NewCenturySchlbk Roman',
+    'Palatino-Bold':                 'Palatino Bold',
+    'Palatino-BoldItalic':           'Palatino Bold Italic',
+    'Palatino-Italic':               'Palatino Italic',
+    'Palatino-Roman':                'Palatino Roman',
+    'ZapfChancery-MediumItalic':     'ZapfChancery Medium Italic',
+    'ZapfDingbats':                  'ZapfDingbats',
+}
+
+
 class TextShape(Shape):
 
     LEFT, CENTER, RIGHT = -1, 0, 1
-    DEFAULT_FONTNAME = Gtk.Settings.get_default().get_property("gtk-font-name")
 
     def __init__(self, pen, x, y, j, w, t):
         Shape.__init__(self)
@@ -112,10 +160,6 @@ class TextShape(Shape):
         self.w = w  # width
         self.t = t  # text
 
-    def _font_available(self, fontname, pango_context):
-        available_fonts = [family.get_name() for family in pango_context.list_families()]
-        return fontname in available_fonts
-
     def _draw(self, cr, highlight, bounding):
 
         try:
@@ -142,9 +186,6 @@ class TextShape(Shape):
                 # https://git.gnome.org/browse/pygobject/commit/?id=b21f66d2a399b8c9a36a1758107b7bdff0ec8eaa
                 pass
 
-            # set font
-            font = Pango.FontDescription()
-
             # https://developer.gnome.org/pango/stable/PangoMarkupFormat.html
             markup = GObject.markup_escape_text(self.t)
             if self.pen.bold:
@@ -164,16 +205,9 @@ class TextShape(Shape):
             assert success
             layout.set_attributes(attrs)
 
-            if self._font_available(self.pen.fontname, pango_context=context):
-                font.set_family(self.pen.fontname)
-            else:
-                msg = "Font family {fontname!r} is not available, using {default!r}".format(
-                    fontname=self.pen.fontname,
-                    default=self.DEFAULT_FONTNAME
-                )
-                warnings.warn(msg)
-                font.set_family(self.DEFAULT_FONTNAME)
-
+            # set font
+            fontname = fontname_map.get(self.pen.fontname, self.pen.fontname)
+            font = Pango.FontDescription(fontname)
             font.set_absolute_size(self.pen.fontsize*Pango.SCALE)
             layout.set_font_description(font)
 
@@ -605,12 +639,13 @@ def square_distance(x1, y1, x2, y2):
 
 class Edge(Element):
 
-    def __init__(self, src, dst, points, shapes, tooltip):
+    def __init__(self, src, dst, points, shapes, tooltip, url):
         Element.__init__(self, shapes)
         self.src = src
         self.dst = dst
         self.points = points
         self.tooltip = tooltip
+        self.url = url
 
     RADIUS = 10
 
@@ -625,9 +660,15 @@ class Edge(Element):
             return True
         if self.is_inside_end(x, y):
             return True
+
+        for shape in self.shapes:
+            min_dist = shape.get_smallest_distance(x, y)
+            if min_dist is not None and min_dist <= self.RADIUS:
+                return True
+
         return False
 
-    def get_jump(self, x, y):
+    def get_jump(self, x, y, to_dst = False):
         for shape in self.shapes:
             x1, y1, x2, y2 = shape.bounding
             if (x1 - self.RADIUS) <= x and x <= (x2 + self.RADIUS) and (y1 - self.RADIUS) <= y and y <= (y2 + self.RADIUS):
@@ -639,10 +680,20 @@ class Edge(Element):
         for shape in self.shapes:
             distance = shape.get_smallest_distance(x, y)
             if distance is not None and distance <= self.RADIUS:
-                return Jump(self, self.src.x, self.src.y)
+                jmp_dest = self.dst if to_dst else self.src
+                return Jump(self, jmp_dest.x, jmp_dest.y)
 
         return None
 
+    def get_url(self, x, y):
+        if self.is_inside_begin(x, y) and self.url['head'] is not None:
+            return Url(self, self.url['head'])
+        if self.is_inside_end(x, y) and self.url['tail'] is not None:
+            return Url(self, self.url['tail'])
+        if self.is_inside(x, y) and self.url['body'] is not None:
+            return Url(self, self.url['body'])
+        return None
+
     def __repr__(self):
         return "<Edge %s -> %s>" % (self.src, self.dst)
 
@@ -722,17 +773,22 @@ class Graph(Shape):
         for edge in self.edges:
             if edge.is_inside(x, y):
                 return edge
+        return None
 
     def get_url(self, x, y):
         for node in self.nodes:
             url = node.get_url(x, y)
             if url is not None:
                 return url
+        for edge in self.edges:
+            url = edge.get_url(x, y)
+            if url is not None:
+                return url
         return None
 
-    def get_jump(self, x, y):
+    def get_jump(self, x, y, to_dst = False):
         for edge in self.edges:
-            jump = edge.get_jump(x, y)
+            jump = edge.get_jump(x, y, to_dst)
             if jump is not None:
                 return jump
         for node in self.nodes:
