@@ -82,6 +82,7 @@ class DotWidget(Gtk.DrawingArea):
 
         self.connect('key-press-event', self.on_key_press_event)
         self.last_mtime = None
+        self.mtime_changed = False
 
         GLib.timeout_add(1000, self.update)
 
@@ -96,6 +97,9 @@ class DotWidget(Gtk.DrawingArea):
         self.history_back = []
         self.history_forward = []
 
+        self.zoom_gesture = Gtk.GestureZoom.new(self)
+        self.zoom_gesture.connect("scale-changed", self.on_scale_changed)
+
     def error_dialog(self, message):
         self.emit('error', message)
 
@@ -150,13 +154,14 @@ class DotWidget(Gtk.DrawingArea):
                 self.last_mtime = None
             else:
                 self.last_mtime = os.stat(filename).st_mtime
+            self.mtime_changed = False
             self.openfilename = filename
             return True
 
     def set_xdotcode(self, xdotcode, center=True):
         assert isinstance(xdotcode, bytes)
 
-        if self.graphviz_version is None:
+        if self.graphviz_version is None and self.filter is not None:
             stdout = subprocess.check_output([self.filter, '-V'], stderr=subprocess.STDOUT)
             stdout = stdout.rstrip()
             mo = re.match(br'^.* - .* version (?P<version>.*) \(.*\)$', stdout)
@@ -186,6 +191,9 @@ class DotWidget(Gtk.DrawingArea):
                 return True
             if current_mtime != self.last_mtime:
                 self.last_mtime = current_mtime
+                self.mtime_changed = True
+            elif self.mtime_changed:
+                self.mtime_changed = False
                 self.reload()
         return True
 
@@ -438,7 +446,8 @@ class DotWidget(Gtk.DrawingArea):
                 if url is not None:
                     self.emit('clicked', url.url, event)
                 else:
-                    jump = self.get_jump(x, y)
+                    ctrl_held = event.state & Gdk.ModifierType.CONTROL_MASK
+                    jump = self.get_jump(x, y, to_dst=ctrl_held)
                     if jump is not None:
                         self.animate_to(jump.x, jump.y)
 
@@ -471,6 +480,13 @@ class DotWidget(Gtk.DrawingArea):
         if self.zoom_to_fit_on_resize:
             self.zoom_to_fit()
 
+    def on_scale_changed(self, gesture, scale):
+        point, x, y = gesture.get_point()
+        if point:
+            pos = (x, y)
+        new_zoom_ratio = self.zoom_ratio * math.exp(math.log(scale) / 3)
+        self.zoom_image(new_zoom_ratio, pos=pos)
+
     def animate_to(self, x, y):
         del self.history_forward[:]
         self.history_back.append(self.get_current_pos())
@@ -523,9 +539,9 @@ class DotWidget(Gtk.DrawingArea):
         x, y = self.window2graph(x, y)
         return self.graph.get_url(x, y)
 
-    def get_jump(self, x, y):
+    def get_jump(self, x, y, to_dst = False):
         x, y = self.window2graph(x, y)
-        return self.graph.get_jump(x, y)
+        return self.graph.get_jump(x, y, to_dst)
 
 
 class FindMenuToolAction(Gtk.Action):
